require "diffy"

module Crosby
  class Diff
    class << self
      def compare(uuid, options = {}, &block)
        new(uuid, options).compare(&block)
      end
    end

    def initialize(uuid, options = {})
      @uuid = uuid
      @config = Crosby.config.merge(options)
    end

    # https://github.com/samg/diffy#supported-formats
    def compare(format = :color, &block)
      diffs.map { |diff|
        out = diff.to_s(format)
        block_given? ? block.call(out) : out
      }
    end

    def diffs
      @diffs ||= [].tap { |diffs|
        pairs.each { |pair|
          diffs << Diffy::Diff.new(
            *pair,
            :context => 0,
            :include_diff_info => true,
            :source => "files"
          )
        }
      }
    end

    private

    def build_file_paths(pairs)
      pairs.map { |pair|
        pair.map { |filename|
          File.realpath(File.join(@config[:export_path], filename))
        }
      }
    end

    def files
      matcher = Regexp.new("#{@uuid}\\.crosby\\z")

      Dir.new(@config[:export_path]).entries.select { |entry|
        entry =~ matcher ? true : false
      }
    end

    def pairs
      [
        :unique_pairs,
        :sort_pairs,
        :build_file_paths
      ].reduce([]) { |obj, meth|
        send(meth, obj)
      }
    end

    def sort_pairs(pairs)
      pairs.map { |pair|
        pair.first =~ Regexp.new(@config[:app_name]) ? pair.reverse : pair
      }
    end

    def unique_pairs(arr)
      arr.tap { |pairs|
        files.repeated_permutation(2) { |permutation|
          pairs << permutation.sort unless permutation.uniq.length == 1
        }

        pairs.uniq!
      }
    end
  end
end
