require "crosby"

module Crosby
  class Tasks
    include Rake::DSL if defined? Rake::DSL

    def install_tasks
      namespace :crosby do
        desc "displays all diffs in the configured folder " +
             "(#{Crosby.config[:export_path]})"
        task :diff do
          each_uuid do |uuid|
            puts "============================================================"
            puts "UUID: #{uuid}"
            puts "============================================================"
            Crosby::Diff.compare(uuid) { |diff| puts diff }
          end
        end
      end
    end

    private

    def each_uuid(&block)
      [].tap { |uuids|
        Dir.new(Crosby.config[:export_path]).each { |filename|
          uuids << extract_uuid(filename) if filename =~ /\.crosby$/
        }

        uuids.compact!
        uuids.uniq!
        uuids.sort!

        uuids.each { |uuid| block.call(uuid) } if block_given?
      }
    end

    def extract_uuid(filename)
      filename.gsub(/^.+\..+\..+\.(\S+)\.crosby$/, '\1')
    end
  end
end

Crosby::Tasks.new.install_tasks
